#include <stdio.h>
#include <stdlib.h>
#include "Graph.h"

#ifdef OSX
#include <string.h>
#endif

/* Returns an initialized g_opts data type. */
Opts* newOpts(int argv, char **argc) {
  Opts  *opts;
  int   arg;
  
  /* create opts */
  opts = malloc(sizeof(Opts));

  /* set defaults */
  opts->type    = "";
  opts->prob    = -1;
  opts->size    = -1;
  opts->links   = -1;
  opts->samples = 1;
  
  opts->useList       = 0;
  opts->useMatrix     = 0;
  opts->useUnionFind  = 0;
  opts->measCluster   = 0;
  opts->measComponent = 0;
  opts->measDegree    = 0;
  opts->measCompDeg   = 0;
  opts->measLength    = 0;
  opts->printDot      = 0;
  opts->printGraphrep = 0;
  
  opts->G = 0;
  

  /* read arguments and set options */
  for (arg = 1; arg < argv; arg++) {
    if ( argc[arg][0] == '-' )
      switch (argc[arg][1]) {
        case 'd':
          arg++;
          if ( strcmp( argc[arg], "list" ) == 0 )
            opts->useList = 1;
          else if ( strcmp ( argc[arg], "matrix" ) == 0)
            opts->useMatrix = 1;
          break;
        case 'k':
          opts->links = atoi(argc[++arg]);
          break;
        case 'm':
          arg++;
          if ( strcmp( argc[arg], "cluster" ) == 0 ) {
            opts->measCluster  = 1;
            opts->useUnionFind = 1;
          }
          else if ( strcmp( argc[arg], "component" ) == 0) {
            opts->measComponent = 1;
            opts->useUnionFind  = 1;
          }
          else if ( strcmp( argc[arg], "degree" ) == 0)
            opts->measDegree  = 1;
          else if ( strcmp( argc[arg], "compdegree") == 0) {
            opts->measCompDeg  = 1;
            opts->useList      = 1;
            opts->useUnionFind = 1;
          }
          else if ( strcmp( argc[arg], "length" ) == 0) {
            opts->measLength   = 1;
            opts->useUnionFind = 1;
            opts->useList      = 1;
          }
          break;
        case 'o':
          arg++;
          if ( strcmp( argc[arg], "dot" ) == 0 ) {
            opts->printDot  = 1;
            opts->useMatrix = 1;
          }
          else if ( strcmp ( argc[arg], "graph" ) == 0)
            opts->printGraphrep = 1;
          break;
        case 'p':
          opts->prob = atof(argc[++arg]);
          break;
        case 's':
          opts->size = atoi(argc[++arg]);
          break;
        case 't':
          opts->type = argc[++arg];
          break;
        case 'x':
          opts->samples = atoi(argc[++arg]);
          break;
        default:
          printOptsUsage(argc[0]);
          break;
      }
  }
  return opts;
}


void printOptsUsage(char *name) {
  printf("\nUSAGE:\n");
  printf("\t%s <-t type> <-s size> <-p prob> [-k link] [options]\n", name);
  printf("\nREQUIRED ARGS: <-t type> <-s size> <-p prob> [-l link]\n");
  printf("\ttype: chkns, erdos, watts, lancaster\n");
  printf("\tsize: number of nodes\n");
  printf("\tprob: probability (different meaning for different models)\n");
  printf("\tlink: number of links to make (required for lancaster & watts models)\n");
  printf("\nOPTIONAL ARGS: [-m meas] [-o outp] [-d data] [-x samp]\n");
  printf("\tmeas: cluster, component, degree, compdegree, length\n");
  printf("\toutp: graph, dot\n");
  printf("\tdata: matrix, list (internal storage type)\n");
  printf("\tsamp: number of trials to average over\n");
  printf("\n");
  exit(1);
}

