#include <stdlib.h>
#include "UnionFind.h"

/* -----------------
 * Private Functions ======================================================
 * ----------------- */

void _union (int A[], int i, int j) {
  if ( A[i] < A[j] ) {
    A[i] += A[j];
    A[j] = i;
  }
  else {
    A[j] += A[i];
    A[i] = j;
  }
  return;
}


int _findRecursive (int A[], int i) {

  if (A[i] < 0)
    return i;

  return A[i] = _findRecursive(A, A[i]);
}


int _findIterative (int A[], int i) {
  int j = i, k;

  /* if this is the leader, just return */
  if (A[i] < 0)
    return i;

  /* find the leader */
  while (A[j] >= 0)
    j = A[j];
  
  /* again, find the leader, but change each node
   * we pass through to point at the leader */
  while (A[i] != j) {
    k = A[i];   /* find next node to go to */
    A[i] = j;   /* make current node point to leader */
    i = k;      /* onto our next node */
  }
  
  return j;
}


#define _find  _findIterative

void _uf_link (UnionFind * uf, int i, int j) {
  int i_leader = _find(uf->array, i), j_leader = _find(uf->array, j);
      
  if (i_leader != j_leader)       /* merge if they are in different sets */
    _union(uf->array, i_leader, j_leader);
}

void _uf_init(UnionFind* uf) {
  int i = 0;
  while ( i < uf->size )
    uf->array[i++] = -1;
}


/* ----------------
 * Public Functions =======================================================
 * ---------------- */

UnionFind newUnionFind(int const size) {
  int i;
  UnionFind uf;

  uf.array = malloc(size * sizeof(int));
  /* initialize array */
  for (i = 0 ; i < size ; i++)
    uf.array[i] = -1;
  uf.size  = size;
  uf.link  = _uf_link;
  uf.init  = _uf_init;

  return uf;
}


void fastlinkUnionFind (UnionFind * uf, int v1, int v2) {
  int i, j, k, v1_lead, v2_lead;
  int *A = uf->array;

  
  j = i = v1;
  /* if this is the leader, just return */
  if (A[i] < 0)
    v1_lead = i;
  else {
    /* find the leader */
    while (A[j] >= 0)
      j = A[j];
    
    /* again, find the leader, but change each node
     * we pass through to point at the leader */
    while (A[i] != j) {
      k = A[i];   /* find next node to go to */
      A[i] = j;   /* make current node point to leader */
      i = k;      /* onto our next node */
    }
    
    v1_lead = j;
  }

  
  j = i = v2;
  /* if this is the leader, just return */
  if (A[i] < 0)
    v2_lead = i;
  else {
    /* find the leader */
    while (A[j] >= 0)
      j = A[j];
    
    /* again, find the leader, but change each node
     * we pass through to point at the leader */
    while (A[i] != j) {
      k = A[i];   /* find next node to go to */
      A[i] = j;   /* make current node point to leader */
      i = k;      /* onto our next node */
    }
    
    v2_lead = j;
  }

  
  /* do nothing if v1 and v2 are in different sets */
  if (v1_lead == v2_lead)
    return;
 
  /* merge if they are in different sets */
  i = v1_lead;
  j = v2_lead;
  if ( A[i] < A[j] ) {
    A[i] += A[j];
    A[j] = i;
  }
  else {
    A[j] += A[i];
    A[i] = j;
  }
  
}
