#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "Graph.h"
#include "measure.h"
#include "output.h"
#include "randgraphs.h"

#ifdef OSX
#include <string.h>
#endif


void checkArgs(int argv, char **argc) {
  if (argv < 7) {
    printOptsUsage(argc[0]);
    exit(1);
  }
}

void initRNG(unsigned int seed) {
  time_t t;
  time(&t);
  (seed) ? srandom(seed) : srandom(t);
}


int main(int argv, char **argc) {
  int     s, comp, *degdist;
  double  clust, pathlen;
  
  Graph   G;
  Opts    *opts;

  checkArgs(argv, argc);
  opts = newOpts(argv, argc);
  initRNG(0);

  comp = clust = pathlen = 0;
  degdist = calloc(opts->size, sizeof(int));
  
  for (s = 0; s < opts->samples; s++) {
    /* generate the graph */
    G = genGraph(opts);
    opts->G = &G;

    /* collect measurements */
    if (opts->measComponent)
      comp   += measGiantComponent(G);
    if (opts->measCluster)
      clust  += measAvgClustCoeff(G);
    if (opts->measDegree)
      measDegDist(G, degdist);
    if (opts->measCompDeg)
      measCompDegDist(G, degdist);
    if (opts->measLength)
      pathlen += measAvgPathLen(G);
    
    /* if there is another iteration, initialize the graph structs */
    if (s + 1 < opts->samples) {
      if (G.hasUnionFind)
        G.unionfind.init(&G.unionfind);
      if (G.hasMatrix)
        G.adjmat.init(&G.adjmat);
      if (G.hasList)
        G.adjlist.init(&G.adjlist);
    }
    
    if (opts->printGraphrep)
      printGraph(G);
  }

  /* output time */
  if (opts->measComponent)
    printf("# Avg giant component size (%% of nodes):\n%g\n", 
      ( comp / (double)opts->samples ) / opts->size );
  if (opts->measCluster)
    printf("# Avg giant component clustering coeffecient:\n%g\n", clust / (double)opts->samples);
  if (opts->measDegree) {
    printf("# Avg degree distribution:\n");
    printDegDist(G.size, opts->samples, degdist);
  }
  if (opts->measCompDeg) {
    printf("# Avg degree distribution of giant component:\n");
    printDegDist(G.size, opts->samples, degdist);
  }
  if (opts->measLength)
    printf("# Average path length:\n%g\n", pathlen / (double)opts->samples);
  if (opts->printGraphrep)
    printGraph(G);
  if (opts->printDot)
    printDot(G);
  
  return 0;
}


