#include "AdjList.h"
#include "measure.h"
#include "output.h"

/* Prints a non-unique list of component sizes in the graph. */
void printComponentSizes(Graph G) {
  int i = 0;
  while ( i < G.unionfind.size )
    if (G.unionfind.array[i] < 0)
      printf("%d\n", -G.unionfind.array[i++]);
}

/* Prints an ASCII representation of the graph. */
void printGraph(Graph G) {
  int       i, j;
  AdjNode*  cur;
  
  if (G.hasList)
    for (i = 0; i < G.size; i++) {
      printf("%d", i);
      for ( cur = G.adjlist.list[i] ; cur != 0 ; cur = cur->next)
        if (cur->id >= 0)
          printf(" -> %d", cur->id);
      printf("\n");
    }

  if (G.hasMatrix)
    for (j = 0; j < G.adjmat.size; j++) {
      for (i = 0; i <= j; i++)
        printf("%d ", G.adjmat.matrix[j][i]);
      printf("\n");
    }
}

/* Output the graph using the graphviz .dot notation. */
void printDot(Graph G) {
  int j,i;

  if (G.hasMatrix) {
    printf("Graph G {\n");
    printf("\tnode [shape=circle,fontcolor=white,fontsize=0,height=0.12,width=0.12]\n");
    for (j = 0; j < G.adjmat.size; j++) {
      printf("\t%d\n", j);
      for (i = 0; i < j; i++)
        if (G.adjmat.matrix[j][i])
          printf("\t%d -- %d\n", j, i);
    }
    printf("}\n");
  }
  else
    printf("Only available for matrix representations.\n");
}

/* Prints the degree distribution. */
void printDegDist(int size, int samps, int *dist) {
  int i = 0;
  for ( ; i < size ; i++)
    if (dist[i])
      printf("%d %g\n", i, dist[i] / (double)samps );
}
