#! /usr/bin/perl -w
#
# A wrapper for rg.
#
# The idea is that you pass the whole ./rg command line
# to this program replacing one of its usual arguments
# with a special symbol this wrapper will interpret.
#
# That special symbol will be replaced with successive
# values indicated via arguments to this script.
#
# Wow, can my description be any more dry?
# 
use strict;
use Getopt::Std;

my %opts;
getopt('uelms', \%opts);

my $meas  = $opts{'m'} or die("Forgot number of measurements.\n");
my $exec  = $opts{'e'} or die("Uhh, i need a command.\n");
my $upper = $opts{'u'} or die("Forgot upper limit.\n");
my $lower = $opts{'l'} or die("Forgot lower limit.\n");
my $int   = $opts{'i'} && 1;
my $start = $opts{'s'} || 1;

my $eps   = ($upper - $lower) / ($meas - 1);
my %results;

my $val   = $lower + ($start - 1) * $eps;

for ($start .. $meas) {
  # collect the current time
  my $timestamp = time;

  # output a nice timestamp so we can eyeball how
  # long this run will take
  my $timestr = `date +%H:%M:%S`;
  chomp $timestr;
  print STDERR "[$timestr] Calculating measurement $_ of $meas.\n";

  my $useval = ($int) ? int($val) : $val;

  # substitute our value for the special symbol  
  my $cmd = $exec;
  $cmd =~ s/%%/$useval/;

  # execute the command
  my @output = `$cmd`;
  my $result = $output[$#output];

  # print result
  print "$useval $result";

  # increment our value
  $val += $eps;

  # sleep if it hasn't been one second
  # -- this is necessary since our models seed their random number
  # -- generator with the current time.
  sleep 1 unless (time - $timestamp);
}
